<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Criteria;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Criteria\Search\SearchCriteria;
use Org\Mxchange\CoreFramework\Object\BaseFrameworkSystem;
use Org\Mxchange\CoreFramework\Utils\Strings\StringUtils;

/**
 * A general crtieria class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2021 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
abstract class BaseCriteria extends BaseFrameworkSystem implements Criteria {
	/**
	 * Frontend class name stored in config entry
	 */
	private $frontendConfigEntry = '';

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct (string $className) {
		// Call parent constructor
		parent::__construct($className);

		// Initialize all criteria arrays
		foreach (array('default', 'choice', 'exclude') as $criteriaType) {
			// Init it
			$this->initGenericArrayKey('criteria', $criteriaType, 'entries');
		} // END - foreach
	}

	/**
	 * Count the criteria, e.g. useful to find out if a database query has no
	 * limitation (search criteria).
	 *
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @return	$count	Count of all criteria entries
	 */
	protected final function count (string $criteriaType = 'default') {
		// Return it
		return $this->countGenericArrayGroup('criteria', $criteriaType);
	}

	/**
	 * Checks whether given key is set
	 *
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @param	$criteriaKey	Criteria key
	 * @return	$isSet			Whether key is set
	 */
	public function isKeySet (string $criteriaType, string $criteriaKey) {
		// Make sure no 'my-' or 'my_' passes this point
		assert((strpos($criteriaKey, 'my-') === false) && (strpos($criteriaKey, 'my_') === false));

		// Determine it
		$isSet = $this->isGenericArrayElementSet('criteria', $criteriaType, 'entries', $criteriaKey);

		// Return it
		return $isSet;
	}

	/**
	 * Checks whether given key is set for 'choice' type
	 *
	 * @param	$criteriaKey	Criteria key
	 * @return	$isSet			Whether key is set
	 */
	public function isChoiceKeySet (string $criteriaKey) {
		// Call inner method
		return $this->isKeySet('choice', $criteriaKey);
	}

	/**
	 * Checks whether given key is set for 'exclude' type
	 *
	 * @param	$criteriaKey	Criteria key
	 * @return	$isSet			Whether key is set
	 */
	public function isExcludeKeySet (string $criteriaKey) {
		// Call inner method
		return $this->isKeySet('exclude', $criteriaKey);
	}

	/**
	 * Setter for frontend class name
	 *
	 * @param	$frontendConfigEntry		Configuration entry which hold the frontend class' name
	 * @return	void
	 */
	public final function setFrontendConfigEntry (string $frontendConfigEntry) {
		$this->frontendConfigEntry = $frontendConfigEntry;
	}

	/**
	 * Getter for frontend class name
	 *
	 * @return	$frontendConfigEntry		Configuration entry which hold the frontend class' name
	 */
	public final function getFrontendConfigEntry () {
		return $this->frontendConfigEntry;
	}

	/**
	 * Getter for criteria array
	 *
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @return	$criteria
	 */
	public final function getCriteriaArray (string $criteriaType = 'default') {
		return $this->getGenericArrayKey('criteria', $criteriaType, 'entries');
	}

	/**
	 * Getter for criteria array 'choice' type
	 *
	 * @return	$criteria
	 */
	public final function getCriteriaChoiceArray () {
		return $this->getCriteriaArray('choice');
	}

	/**
	 * Getter for criteria array 'exclude' type
	 *
	 * @return	$criteria
	 */
	public final function getCriteriaExcludeArray () {
		return $this->getCriteriaArray('exclude');
	}

	/**
	 * Unsets a criteria key from all criteria types
	 *
	 * @param	$criteriaKey	Criteria key to unset
	 * @return	void
	 */
	public final function unsetCriteria (string $criteriaKey) {
		// Make sure no 'my-' or 'my_' passes this point
		assert((strpos($criteriaKey, 'my-') === false) && (strpos($criteriaKey, 'my_') === false));

		// Convert dashes to underscore
		$criteriaKey = StringUtils::convertDashesToUnderscores($criteriaKey);

		// "Walk" through all criterias
		foreach ($this->getGenericArray('criteria') as $criteriaType => $dummy) {
			// Remove it
			$this->unsetGenericArrayElement('criteria', $criteriaType, 'entries', $criteriaKey);
		} // END - foreach
	}

	/**
	 * Add criteria, this method converts dashes to underscores because dashes
	 * are not valid for criteria keys.
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$criteriaValue	Criteria value
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @return	void
	 */
	public final function addCriteria (string $criteriaKey, $criteriaValue, string $criteriaType = 'default') {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(strtoupper($criteriaType) . '-CRITERIA: criteriaKey=' . $criteriaKey . ',criteriaValue=' . $criteriaValue . ',criteriaType=' . $criteriaType . ' - CALLED!');

		// Make sure no 'my-' or 'my_' passes this point
		assert((strpos($criteriaKey, 'my-') === false) && (strpos($criteriaKey, 'my_') === false) && (!is_bool($criteriaValue)));

		// Convert dashes to underscore
		$criteriaKey = StringUtils::convertDashesToUnderscores($criteriaKey);

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(strtoupper($criteriaType) . '(' . $this->__toString() . ')-CRITERIA: criteriaKey=' . $criteriaKey);

		// Append it
		$this->appendStringToGenericArrayElement('criteria', $criteriaType, 'entries', $criteriaKey, $criteriaValue);
	}

	/**
	 * Set criteria, this method converts dashes to underscores because dashes
	 * are not valid for criteria keys.
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$criteriaValue	Criteria value
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @return	void
	 */
	public final function setCriteria (string $criteriaKey, $criteriaValue, string $criteriaType = 'default') {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(strtoupper($criteriaType) . '-CRITERIA: criteriaKey=' . $criteriaKey . ',criteriaValue=' . $criteriaValue . ',criteriaType=' . $criteriaType . ' - CALLED!');

		// Make sure no 'my-' or 'my_' passes this point
		assert((strpos($criteriaKey, 'my-') === false) && (strpos($criteriaKey, 'my_') === false) && (!is_bool($criteriaValue)));

		// Convert dashes to underscore
		$criteriaKey = StringUtils::convertDashesToUnderscores($criteriaKey);

		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(strtoupper($criteriaType) . '(' . $this->__toString() . ')-CRITERIA: criteriaKey=' . $criteriaKey);

		// Set it
		$this->setStringGenericArrayElement('criteria', $criteriaType, 'entries', $criteriaKey, $criteriaValue);
	}

	/**
	 * Add "choice" criteria, this method converts dashes to underscores because
	 * dashes are not valid for criteria keys.
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$criteriaValue	Criteria value
	 * @return	void
	 */
	public final function addChoiceCriteria (string $criteriaKey, $criteriaValue) {
		// Make sure no 'my-' or 'my_' passes this point
		assert((strpos($criteriaKey, 'my-') === false) && (strpos($criteriaKey, 'my_') === false) && (!is_bool($criteriaValue)));

		// Add it
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(strtoupper($criteriaType) . '(' . $this->__toString() . ')-CRITERIA: criteriaKey=' . $criteriaKey . ',criteriaValue=' . $criteriaValue);
		$this->pushValueToGenericArrayElement('criteria', 'choice', 'entries', StringUtils::convertDashesToUnderscores($criteriaKey), (string) $criteriaValue);
	}

	/**
	 * Add "exclude" criteria, this method converts dashes to underscores because
	 * dashes are not valid for criteria keys.
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$criteriaValue	Criteria value
	 * @return	void
	 */
	public final function addExcludeCriteria (string $criteriaKey, $criteriaValue) {
		// Add it with generic method
		$this->addCriteria($criteriaKey, $criteriaValue, 'exclude');
	}

	/**
	 * Add configured criteria
	 *
	 * @param	$criteriaKey	Criteria key
	 * @param	$configEntry	Configuration entry
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @return	void
	 */
	public final function addConfiguredCriteria (string $criteriaKey, string $configEntry, string $criteriaType = 'default') {
		// Add the configuration entry as a criteria
		$value = FrameworkBootstrap::getConfigurationInstance()->getConfigEntry($configEntry);
		$this->addCriteria($criteriaKey, $value, $criteriaType);
	}

	/**
	 * Get criteria element or false if not found
	 *
	 * @param	$criteriaKey	The requested criteria key
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @return	$value			Whether the value of the critera or false
	 */
	public function getCriteriaElemnent (string $criteriaKey, string $criteriaType = 'default') {
		// Make sure no 'my-' or 'my_' passes this point
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(strtoupper($criteriaType) . '-CRITERIA: criteriaKey=' . $criteriaKey . ',criteriaType=' . $criteriaType . ' - CALLED!');
		assert((strpos($criteriaKey, 'my-') === false) && (strpos($criteriaKey, 'my_') === false));

		// Convert dashes to underscore
		$criteriaKey = StringUtils::convertDashesToUnderscores($criteriaKey);

		// Default is not found
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(strtoupper($criteriaType) . '-CRITERIA: criteriaKey=' . $criteriaKey . ',criteria()=' . $this->countGenericArrayGroup('criteria', $criteriaType));
		$value = false;

		// Is the criteria there?
		if ($this->isKeySet($criteriaType, $criteriaKey)) {
			// Then use it
			$value = $this->getGenericArrayElement('criteria', $criteriaType, 'entries', $criteriaKey);
		} // END - if

		// Return the value
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput(strtoupper($criteriaType) . '-CRITERIA: value=' . $value . ' - EXIT!');
		return $value;
	}

	/**
	 * Get criteria element or false if not found for 'choice' type
	 *
	 * @param	$criteriaKey	The requested criteria key
	 * @return	$value			Whether the value of the critera or false
	 */
	public function getCriteriaChoiceElemnent (string $criteriaKey) {
		// Call inner method
		return $this->getCriteriaElemnent($criteriaKey, 'choice');
	}

	/**
	 * Get criteria element or false if not found for 'exclude' type
	 *
	 * @param	$criteriaKey	The requested criteria key
	 * @return	$value			Whether the value of the critera or false
	 */
	public function getCriteriaExcludeElemnent (string $criteriaKey) {
		// Call inner method
		return $this->getCriteriaElemnent($criteriaKey, 'exclude');
	}

	/**
	 * Checks whether given array entry matches
	 *
	 * @param	$entryArray		Array with the entries to find
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @return	$matches		Whether the entry matches or not
	 */
	public function ifEntryMatches (array $entryArray, string $criteriaType = 'default') {
		// First nothing matches and nothing is counted
		$matches = false;
		$counted = 0;

		// Walk through all entries
		foreach ($entryArray as $key => $entry) {
			// Make sure no 'my-' or 'my_' passes this point
			assert((strpos($key, 'my-') === false) && (strpos($key, 'my_') === false));

			// Convert dashes to underscore
			$key = StringUtils::convertDashesToUnderscores($key);

			// Then walk through all search criteria
			foreach ($this->getGenericArrayKey('criteria', $criteriaType, 'entries') as $criteriaKey => $criteriaValue) {
				// Make sure no 'my-' or 'my_' passes this point
				assert((strpos($criteriaKey, 'my-') === false) && (strpos($criteriaKey, 'my_') === false) && (!is_bool($criteriaValue)));

				// Convert dashes to underscore
				$criteriaKey = StringUtils::convertDashesToUnderscores($criteriaKey);

				// Is the element found and does it match?
				if (($key == $criteriaKey) && ($criteriaValue == $entry)) {
					// Then count this one up
					$counted++;
				} // END - if
			} // END - foreach
		} // END - foreach

		// Now check if expected criteria counts match
		$matches = ($counted == $this->countGenericArrayGroup('criteria', $criteriaType));

		// Return the result
		return $matches;
	}

	/**
	 * Checks whether given array 'choice' entry matches
	 *
	 * @param	$entryArray		Array with the entries to find
	 * @return	$matches		Whether the entry matches or not
	 */
	public function ifChoiceMatches (array $entryArray) {
		// Call inner method
		return $this->ifEntryMatches($entryArray, 'choice');
	}

	/**
	 * Checks whether given array 'exclude' entry matches
	 *
	 * @param	$entryArray		Array with the entries to find
	 * @return	$matches		Whether the entry matches or not
	 */
	public function ifExcludeMatches (array $entryArray) {
		// Call inner method
		return $this->ifEntryMatches($entryArray, 'exclude');
	}

	/**
	 * "Getter" for a cache key
	 *
	 * @param	$onlyKeys	Only use these keys for a cache key
	 * @param	$criteriaType	Type of this criteria, can be one of 'default' (default), 'choice' or 'exclude'
	 * @return	$cacheKey	The key suitable for the cache system
	 */
	public function getCacheKey (array $onlyKeys = [], string $criteriaType = 'default') {
		// Debug message
		//* NOISY-DEBUG: */ self::createDebugInstance(__CLASS__, __LINE__)->debugOutput($this->__toString() . ': criteriaType=' . $criteriaType . ',count()=' . $this->countGenericArray('criteria')));

		// Make sure the criteria is there
		assert($this->isValidGenericArrayGroup('criteria', $criteriaType));

		// Initialize the key
		$cacheKey = '';

		// Now walk through all criterias
		foreach ($this->getGenericArrayKey('criteria', $criteriaType, 'entries') as $criteriaKey => $criteriaValue) {
			// Make sure no 'my-' or 'my_' passes this point
			assert((strpos($criteriaKey, 'my-') === false) && (strpos($criteriaKey, 'my_') === false) && (!is_bool($criteriaValue)));

			// $criteriaValue cannot be an array
			assert(!is_array($criteriaValue));

			// Convert dashes to underscore
			$criteriaKey = StringUtils::convertDashesToUnderscores($criteriaKey);

			// Is the value in array or is $onlyKeys empty?
			if ((isset($onlyKeys[$criteriaKey])) || (count($onlyKeys) == 0)) {
				// Add the value URL encoded to avoid any trouble with special characters
				$cacheKey .= sprintf('%s=%s;',
					$criteriaKey,
					urlencode($criteriaValue)
				);
			} // END - if
		} // END - foreach

		// Remove last semicolon
		$cacheKey = substr($cacheKey, 0, -1);

		// Is the instance SearchCriteria?
		if ($this instanceof SearchCriteria) {
			// Check if 'limit' and 'skip' are in
			if (((isset($onlyKeys['limit'])) && (isset($onlyKeys['skip']))) || (count($onlyKeys) == 0)) {
				// Add limit and skip values
				$cacheKey .= sprintf(';%%limit%%=%s;%%skip%%=%s',
					$this->getLimit(),
					$this->getSkip()
				);
			} // END - if
		} // END - if

		// Return the cache key
		return $cacheKey;
	}

	/**
	 * "Getter" for a cache key ('choice' type)
	 *
	 * @param	$onlyKeys	Only use these keys for a cache key
	 * @return	$cacheKey	The key suitable for the cache system
	 */
	public function getCacheKeyChoice (array $onlyKeys = []) {
		// Call inner method
		return $this->getCacheKey($onlyKeys, 'choice');
	}

	/**
	 * "Getter" for a cache key ('exclude' type)
	 *
	 * @param	$onlyKeys	Only use these keys for a cache key
	 * @return	$cacheKey	The key suitable for the cache system
	 */
	public function getCacheKeyExclude (array $onlyKeys = []) {
		// Call inner method
		return $this->getCacheKey($onlyKeys, 'exclude');
	}

	/**
	 * Count 'choice' criteria, e.g. useful to find out if a database query
	 * has no limitation (search criteria).
	 *
	 * @return	$count	Count of all criteria entries
	 */
	public final function countChoice () {
		return $this->count('choice');
	}

	/**
	 * Count 'exclude' criteria, e.g. useful to find out if a database query
	 * has no limitation (search criteria).
	 *
	 * @return	$count	Count of all criteria entries
	 */
	public final function countExclude () {
		return $this->count('exclude');
	}

}
