<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Output\Debug;

// Import framework stuff
use Org\Mxchange\CoreFramework\Debug\Debugger;
use Org\Mxchange\CoreFramework\Generic\UnsupportedOperationException;
use Org\Mxchange\CoreFramework\Output\Debug\BaseDebugOutput;
use Org\Mxchange\CoreFramework\Stream\Output\OutputStreamer;

/**
 * A debug output class for PHP's error_log() command
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2021 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class DebugErrorLogOutput extends BaseDebugOutput implements Debugger, OutputStreamer {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class
	 *
	 * @return	$debugInstance	The prepared debug instance
	 */
	public static final function createDebugErrorLogOutput () {
		// Get a new instance
		$debugInstance = new DebugErrorLogOutput();

		// Return it
		return $debugInstance;
	}

	/**
	 * Outputs the given data without HTML tags, ignores $stripTags
	 *
	 * @param	$output		The HTML'ed output
	 * @param	$stripTags	Whether HTML tags shall be stripped out
	 * @return	void
	 */
	public final function outputStream (string $output, bool $stripTags = false) {
		// Split multiple lines into and array to put them out line-by-line
		$errorLines = explode(chr(10), $output);

		// "Walk" through all lines
		foreach ($errorLines as $err) {
			// Trim any spaces, \n, \r etc. out
			$err = trim($err);

			// Log only none-empty lines
			if (!empty($err)) {
				// Log this line
				error_log(html_entity_decode(strip_tags($err)), 0);
			} // END - if
		} // END - foreach
	}

	/**
	 * Output the code
	 *
	 * @param	$outStream	Stream to output
	 * @param	$stripTags	Whether HTML tags shall be stripped out
	 * @return	void
	 */
	public final function output (string $outStream = '', bool $stripTags = false) {
		// Empty output will be silently ignored
		if (!empty($outStream)) {
			$this->outputStream($outStream);
		}
	}

	/**
	 * Streams the data and maybe does something to it
	 *
	 * @param	$data	The data (string mostly) to "stream"
	 * @return	$data	The data (string mostly) to "stream"
	 * @throws	UnsupportedOperationException	If this method is called
	 */
	public function streamData (string $data) {
		self::createDebugInstance(__CLASS__, __LINE__)->debugOutput('Unhandled ' . strlen($data) . ' bytes in this stream.');
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}

}
